import time
import traceback

from petisco.test_tools.deployment_test_inventory import deploy_test_inventory


class DeployTestsExecutor:
    def __init__(self, verbose: bool = False):
        self.verbose = verbose

    @staticmethod
    def get_available_tests():
        return list(deploy_test_inventory.keys())

    def execute(self, selected_test=None):
        errors = {}
        start_time = time.time()
        is_success = False

        if selected_test and (selected_test in deploy_test_inventory):
            tests = {selected_test: deploy_test_inventory[selected_test]}
        else:
            tests = deploy_test_inventory

        total_tests = len(tests)

        for i, (test_name, deploy_test) in enumerate(tests.items()):

            status = "FAILED"
            try:
                result = deploy_test.execute(verbose=self.verbose)
                if result.is_success:
                    status = "PASSED"
                else:
                    errors[test_name] = result.value.__repr__

            except Exception:
                errors[test_name] = traceback.format_exc()

            print(
                "{test_name:<50} {status} {i:>10}/{total_tests}".format(
                    test_name=test_name, status=status, i=i + 1, total_tests=total_tests
                )
            )
        elapsed_time = time.time() - start_time
        num_errors = len(errors)
        num_passed = total_tests - num_errors

        skipped_tests = len(deploy_test_inventory) - total_tests

        if num_errors > 0:
            print(
                f"Failed {num_errors} of {total_tests} in {elapsed_time:02f} seconds. Skipped {skipped_tests}."
            )
            print("\nFailed Deploy Tests:")
            for test_name, error in errors.items():
                print(
                    "> {test_name}:\n {error}".format(test_name=test_name, error=error)
                )

        else:
            print(
                f"Passed {num_passed} of {total_tests} in {elapsed_time:02f} seconds. Skipped {skipped_tests}."
            )

        if len(errors) == 0:
            is_success = True
        return is_success



from petisco.test_tools import DeployTestBase


class _DeployTestInventory(dict):
    """
    This class implements an inventory that automatically registers classes and functions into a dictionary.
    """

    def add_to(self, container):
        """
        Decorator that automatically adds an asset to the inventory
        Parameters
        ----------
        container: (string) name of inventory entry
        alias: string name to find the asset in the inventory

        Returns
        -------

        """
        if container not in self:

            if not issubclass(container, DeployTestBase):
                raise TypeError(
                    "Only DeployTest objects can be added with the @test_tools decorator"
                )

            self[container.__name__] = container()


deploy_test_inventory = _DeployTestInventory()
deploy_test = deploy_test_inventory.add_to



from petisco.application.singleton import Singleton


class SqlAlchemyPersistence(metaclass=Singleton):
    @staticmethod
    def get_instance():
        return SqlAlchemyPersistence()

    def __init__(self, session=None, base=None):
        self.session = session
        self.base = base



from abc import ABCMeta, abstractmethod

from meiga import Result


class DeployTestBase(object):

    __metaclass__ = ABCMeta

    @abstractmethod
    def execute(self, verbose: bool = False) -> Result:
        raise NotImplementedError



from .deploy_test_base import DeployTestBase
from .deploy_test_executor import DeployTestsExecutor
from .deployment_test_inventory import deploy_test

__all__ = ["DeployTestBase", "DeployTestsExecutor", "deploy_test"]
